//******************************************************************
//* ANAHEIM AUTOMATION BLDC MOTOR DRIVER ROUTINES
//* WRITTEN BY FRED EADY FOR SERVO MAGAZINE
//* LAST UPDATED 1/20/2009
//*
//* THIS CODE WILL SPIN THE MOTOR IN AN OPEN LOOP CONDITION.
//* TO SEE HOW VOLTAGE VERSUS SPEED WORKS PUT A SCOPE PROBE ON
//* THE VC TAP AND RUN THIS CODE IN DEBUG MODE.  BE CAREFUL TO
//* POWER OFF THE +12 POWER WHEN THE CODE IS NOT RUNNING AS YOU
//* MAY SMOKE THE MOSFETS AND DRIVERS. TO RUN THE MOTOR IN A STEADY
//* STATE DON'T RUN THE UPDATE CODE IN THE START MODE. RAMP UP TO A
//* VOLTAGE VALUE OF 576 DECIMAL WITH A COMMUTATION VALUE OF 0x5000.
//* USE THE MPLAB DEBUG WINDOWS TO SEE THE VARIABLES. THE PUSHBUTTONS
//* START AND STOP THE MOTOR.
//* LOOK FOR UPDATES TO THIS CODE AT WWW.EDTP.COM...
//*
//* CHANGES/ADDITIONS
//*
//******************************************************************
#include <pic18.h>
#include <stdio.h>
#include <ctype.h>
#include <htc.h>

// Configuration Fuse Settings for PIC18F2431
__CONFIG(1,IESODIS & FCMDIS & HSPLL);
__CONFIG(2,BORDIS & PWRTDIS & WDTDIS);
__CONFIG(3,PWMOUTDIS & LOWPOLLO & HIPOLLO & MCLREN);
__CONFIG(4,DEBUGEN & LVPDIS & STVREN);
__CONFIG(5,UNPROTECT);
__CONFIG(6,WRTEN);
__IDLOC(0120);

//******************************************************************
//*	FUNCTION PROTOTYPES
//******************************************************************
void init(void);
void pwm_control(char cmd);
void commutate(void);
void mode_service(void);
//******************************************************************
//    MACROS AND PORT/PIN DEFINITIONS
//******************************************************************
#define bitset(var, bitno) ((var) |= 1 << (bitno))
#define bitclr(var, bitno) ((var) &= ~(1 << (bitno)))
#define make8(var,offset)	((unsigned int)var >> (offset * 8)) & 0x00FF
#define	make16(varhigh,varlow)	(((unsigned int)varhigh & 0xFF)* 0x100) + ((unsigned int)varlow & 0x00FF)
#define make32(var1,var2,var3,var4) \
		((unsigned long)var1<<24)+((unsigned long)var2<<16)+ \
		((unsigned long)var3<<8)+((unsigned long)var4)		
#define make32i(var1,var2) ((unsigned long)var1<<16)+((unsigned long)var2)


#define ALIGNMODE_LOOPS 255		// align time in 10ms increments
//******************************************************************
//*	GLOBAL VARIABLES
//******************************************************************

enum{
	STOPMODE,
	ALIGNMODE,
	STARTMODE,
	RUNMODE
};
char x,comm_ptr,mode,comm_slice,tenms_counter;
unsigned int tmsecs1,imsecs1,speedctr,motor_ref,adc_result;
unsigned int delay,voltage,tmr5val,oldtmr5val;
//******************************************************************
//*	GLOBAL DEFINITIONS
//******************************************************************
#define	VM		RA0
#define	SPD_REF	RA1
#define	PHASE_A	RA2
#define PHASE_B	RA3
#define	PHASE_C	RA4

#define	SW1			RC0
#define SW_START 	RC0
#define FAULT_I		RC1
#define SW2			RC2
#define SW_STOP		RC2
#define	LED1		LATC3
#define LED2		LATC4
#define LED3		LATC5
#define	actled1		LATC5
#define	OFF			0
#define ON			1
				
#define	UPDATE_EN	UDIS = 0
#define UPDATE_DIS	UDIS = 1
//******************************************************************
//*	COMMUTATION PATTERNS
//******************************************************************
//*                           +12-GND   PHASES USED  PHASE OPEN
//*#define pos0	0b00000110	;PWM1-PWM2	 	A-B			 C
//*#define pos1	0b00010010	;PWM1-PWM4		A-C			 B
//*#define pos2	0b00011000	;PMW3-PWM4		B-C			 A
//*#define pos3	0b00001001	;PMW3-PWM0		B-A			 C
//*#define pos4	0b00100001	;PWM5-PWM0		C-A			 B
//*#define pos5	0b00100100	;PWM5-PWM2		C-B			 A
char comm_sequence[6] = 	{0x06,0x12,0x18,0x09,0x21,0x24};
char comm_openphase[6] =	{0x01,0x0D,0x09,0x01,0x0D,0x09};
	
//******************************************************************
//*	A2D DEFINITIONS
//******************************************************************
//* VM 		= RA0 = GrpA
//* SPD_REF = RA1 = GrpB
//* PHASE_A = RA2 = GrpC
//* PHASE_B = RA3 = GrpD
//* PHASE_C = RA4 = GrpA
//LOAD THESE TO ADCHS
#define	sel_VM		0b00000000;	//select AN0 instead of AN4 for GrpA
#define sel_PHASE_C 0b00000001; //select AN4 instead of AN0 for GrpA
//LOAD THESE INTO ADCON0
#define	get_VM		0b00000001; //sample and convert GrpA
#define	get_SPD_REF	0b00000101; //sample and convert GrpB
#define	get_PHASE_A	0b00001001; //sample and convert GrpC
#define	get_PHASE_B	0b00001101; //sample and convert GrpD
#define	get_PHASE_C	0b00000001; //sample and convert GrpA
//******************************************************************
//*	INTERRUPT DEFINITIONS
//******************************************************************
#define		disable_TMR5int  	TMR5IE = 0
#define		enable_TMR5int	 	TMR5IE = 1
#define		disable_TMR2int  	TMR2IE = 0
#define		enable_TMR2int	 	TMR2IE = 1
#define		disable_TMR1int  	TMR1IE = 0
#define		enable_TMR1int	 	TMR1IE = 1
#define		disable_TMR0int  	TMR0IE = 0
#define		enable_TMR0int	 	TMR0IE = 1

#define		enable_GLOBALint  	IPEN = 1;				\
							  	GIE = 1;				\
							  	PEIE = 1;
#define		disable_GLOBALint 	IPEN = 1;				\
							  	GIE = 0;				\
							  	PEIE = 0;
#define		TIMER0ON			TMR0ON = 1
#define		TIMER0OFF			TMR0ON = 0
#define		TIMER1ON			TMR1ON = 1
#define		TIMER1OFF			TMR1ON = 0
#define		TIMER2ON		  	TMR2ON = 1
#define		TIMER2OFF		  	TMR2ON = 0
#define		TIMER5ON		  	TMR5ON = 1
#define		TIMER5OFF		  	TMR5ON = 0

//******************************************************************
//*	TIMER5 INTERRUPT HANDLER ROUTINE
//******************************************************************
void interrupt TIMER5(void)
{
		TMR5IF = 0;
		switch(comm_slice)
		{
			case 0:
			{
				commutate();
			}
			break;
			case 1:
				while(GODONE);
				adc_result = make16(ADRESH,ADRESL);
				if(mode == ALIGNMODE)
				{
					comm_slice == 1;
				}
				else
				{
					comm_slice = 0;
				}
			break;
		}
		mode_service();
}
//******************************************************************
//*	TIMER1 INTERRUPT HANDLER ROUTINE
//******************************************************************
void interrupt low_priority TIMER1(void)
{
		TMR1IF = 0;
		TMR1H = 0xEC;	// 1msec@20MHz	
		TMR1L = 0x77;

		++tmsecs1;
		if(++imsecs1 == 1000)
		{
			imsecs1 = 0;
			actled1 ^= 1;
			
		}
}
//******************************************************************
//*	INITIALIZE ROUTINE
//******************************************************************
void init(void)
{
   	TRISA = 0b11111111;		
   	TRISB = 0b11000000;	
   	LATB = 0xFF;	
   	TRISC = 0b11000111;
//******************************************************************
//*	CONFIGURE EEPROM READ/WRITE
//******************************************************************
	EECON1 = 0b00000000;
//******************************************************************
//*	CONFIGURE A2D 
//******************************************************************
	ADCON0 = 0b00000000;	//turn OFF A2D
	ADCON1 = 0b00000000;	//
	ADCON2 = 0b10111111;	//right justified-16TAD-FRC
	ADCON3 = 0b11000010;	//timer5 triggers A2D
	ADCHS  = sel_PHASE_C;	//GrpA-AN0,GrpB-AN1,GrpC-AN2,GrpD-AN3
	ANSEL0 = 0b00011111;	//RA0-RA4 = analog inputs
	ADCON0 = 0b00000001;	//turn ON A2D
//******************************************************************
//*	CONFIGURE AND STARTMODE TIMER1
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	TIMER1OFF;
	T1CON = 0b00000000;
	TMR1H = 0xEC;
	TMR1L = 0x77;
	TMR1IP = 0;
	TIMER1ON;
//******************************************************************
//*	CONFIGURE TIMER5
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	TIMER5OFF;
	T5CON = 0b00000000;
	TMR5H = 0x00;
	TMR5L = 0x00;
	TMR5IP = 1;
//******************************************************************
//*	CONFIGURE PWM 
//******************************************************************
	pwm_control(OFF);
	FLTCONFIG 	= 0b10000000;	//set bit 7 when debugging100001011
	PTCON0		= 0b00000000;
	PTCON1		= 0b00000000;
	PWMCON1 	= 0b00000001;
	PTPERH		= 0x00;
	PTPERL		= 0xFA;
	OVDCOND = 0b00000000;
	OVDCONS = 0b00000000;
  	PDC0H = 0x00;
  	PDC0L = 0x00;
  	PDC1H = 0x00;
  	PDC1L = 0x00;
  	PDC2H = 0x00;
  	PDC2L = 0x00;
//******************************************************************
//*	KEEP HOUSE
//******************************************************************
  	LED1 = OFF;
  	LED2 = OFF;
  	speedctr = 0;
//******************************************************************
//*	ACTIVATE INTERRUPTS
//******************************************************************
	enable_TMR1int;
	disable_TMR5int;
    enable_GLOBALint;
}
//******************************************************************
//*	COMMUTATE
//******************************************************************
void commutate(void)
{
	OVDCOND = comm_sequence[comm_ptr];
	ADCON0 = comm_openphase[comm_ptr];
	++comm_ptr;
	
	if(comm_ptr > 5)
	{
		comm_ptr = 0;
		LED1 ^= 1;
	}
	++comm_slice;
}
//******************************************************************
//*	PWM CONTROL
//******************************************************************
void pwm_control(char cmd)
{
	switch (cmd)
	{
		case ON:
			LATB = 0xFF;
	     	PDC0H = 0x00;
			PDC0L = 0x00;
			PDC1H = 0x00;
			PDC1L = 0x00;
			PDC2H = 0x00;
			PDC2L = 0x00;
			PWMCON0 = 0b01010111;
			PWMCON1 = 0b00000001;
			PTEN = 1;
		break;
		case OFF:
			PTEN = 0;				
			PWMCON0 = 0b00000111;
			OVDCOND = 0b00000000;	
			LATB = 0xFF;
	     	PDC0H = 0x00;
			PDC0L = 0x00;
			PDC1H = 0x00;
			PDC1L = 0x00;
			PDC2H = 0x00;
			PDC2L = 0x00;
		break;
	}
}
//******************************************************************
//*	ENTER MODE
//******************************************************************
void enter_mode(char new_mode)
{
	switch(new_mode)
	{
		case STOPMODE:
			mode = STOPMODE;
			pwm_control(OFF);
			LED2 = OFF;
		break;
		
		case ALIGNMODE:
			mode = ALIGNMODE;	//define the operating mode
			pwm_control(OFF);	//disable the PWMs
			disable_TMR5int;	//no interrupts for TMR5
			TIMER5OFF;			//TMR5 OFF
			TMR5H = 0x00;		//TMR5 counts up to PR5 value
			TMR5L = 0x00;
			PR5H = 0x13;		//load TMR5 for 1mS periods
			PR5L = 0x88;
			motor_ref = 0;
			tenms_counter = 0;
			voltage = 0;		//PWM voltage = 0
			delay = 0;			//init align delay counter
			LED2 = ON;			//for us humans
	  		OVDCOND = comm_sequence[5];	//set commutation state
			ADCHS = sel_VM;
	  		ADCON0 = get_VM; 	//set motor_ref value for A2D
	  		comm_slice = 1;		//tell TMR5 interrupt not to commutate
			pwm_control(ON);	//enable the PWMs
			enable_TMR5int;		//kick off TMR5 1mS intervals
			TIMER5ON;			//TMR5 ON
		break;
		
		case STARTMODE:
			mode = STARTMODE;
		   	comm_slice = 0; 	 //commutate on next TMR5 interrupt
		   	LED2 = OFF;  		 //indicate exit of alignment routine
		   	voltage = 0;         //reset PWM voltage value
			tmr5val = 0x5000;	//3d09 7f09 9f09 a000
			tenms_counter = 0;
			oldtmr5val = tmr5val;
			disable_TMR5int;	//kill TMR5 and its interrupt
			TIMER5OFF;
			TMR5H = 0x00;		//init TMR5 up counter
			TMR5L = 0x00;
			PR5H = make8(tmr5val,1);//TMR5 counts up to PR5 value
			PR5L = make8(tmr5val,0);
			ADCHS = sel_PHASE_C;
		   	enable_TMR5int;		//enable TMR5 and its interrupt
		   	TIMER5ON;
		break;
		
		case RUNMODE:
			mode = RUNMODE;
		break;
	}
}
//******************************************************************
//*	MODE SERVICE ROUTINE
//******************************************************************
void mode_service(void)
{
	switch(mode)
	{
		case STOPMODE:
			pwm_control(OFF);
			comm_slice = 1;
			
		break;
		
//******************************************************************
//*	ALIGN MODE 
//******************************************************************
		case ALIGNMODE:
				if(++tenms_counter >= 10)		//has 10mS passed?
				{
				if(voltage <= 560)				//ramp voltage to 50%+
				{
					voltage += 5;				//increment 5 PWM counts/sec
			     	PDC0H = make8(voltage,1);	//load new PWM values
					PDC0L = make8(voltage,0);
					PDC1H = make8(voltage,1);
					PDC1L = make8(voltage,0);
					PDC2H = make8(voltage,1);
					PDC2L = make8(voltage,0);

				}
				else if (delay < ALIGNMODE_LOOPS)//align delay
				{
					++delay;
				}
				else
				{
				   motor_ref = adc_result;
				   enter_mode(STARTMODE);  		//enter next state
				}
				tenms_counter = 0;				//clear the 10mS counter
			}
		break;
		
//******************************************************************
//*	START MODE 
//******************************************************************
		case STARTMODE:
					if(voltage < 576)
					{
						++voltage;
				     	PDC0H = make8(voltage,1);
						PDC0L = make8(voltage,0);
						PDC1H = make8(voltage,1);
						PDC1L = make8(voltage,0);
						PDC2H = make8(voltage,1);
						PDC2L = make8(voltage,0);
					}
					if(voltage >= 576)
					{
						if(++speedctr == 100)
						{
							speedctr = 0;
							tmr5val -= 100;	//3d09 7f09 9f09 a000
							if((oldtmr5val - tmr5val) >= 0x0800)
							{
								oldtmr5val = tmr5val;
								if(voltage < 579)
								{
									++voltage;
									PDC0H = make8(voltage,1);
									PDC0L = make8(voltage,0);
									PDC1H = make8(voltage,1);
									PDC1L = make8(voltage,0);
									PDC2H = make8(voltage,1);
									PDC2L = make8(voltage,0);
								}
							}
								
							disable_TMR5int;	//kill TMR5 and its interrupt
							TIMER5OFF;
							TMR5H = 0x00;		//init TMR5 up counter
							TMR5L = 0x00;
							PR5H = make8(tmr5val,1);//TMR5 counts up to PR5 value
							PR5L = make8(tmr5val,0);
							enable_TMR5int;		//enable TMR5 and its interrupt
						   	TIMER5ON;
						}
					}
							
					
					
		break;
		
		case RUNMODE:
		break;
	}
}
void main(void)
{
//******************************************************************
//*	INITIALIZE
//******************************************************************

	
  	init();
  	enter_mode(STOPMODE);
  	do{
	  	if(SW_START == 0 && SW_STOP == 1)
	  	{
		  	tmsecs1 = 0;
		  	while(tmsecs1 < 10);
		  	if(SW_START == 0)
		  	{
			  	while(SW_START == 0);
			  	tmsecs1 = 0;
			  	while(tmsecs1 < 10);
			  	if(SW_START == 1)
			  	{
				  	enter_mode(ALIGNMODE);
				}
			}
		}
                                                                                                                                                                                                                                             	  	if(SW_STOP == 0 && SW_START == 1)
	  	{
		  	tmsecs1 = 0;
		  	while(tmsecs1 < 10);
		  	if(SW_STOP == 0)
		  	{
			  	while(SW_STOP == 0);
			  	tmsecs1 = 0;
			  	while(tmsecs1 < 10);
			  	if(SW_STOP == 1)
			  	{
				  	enter_mode(STOPMODE);
				}
			}
		}

	}while(1);

 }

